
import 'package:flustars/flustars.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:tplib/widgets/simple_button.dart';
import 'res_utils.dart';

/**
 * author：lx
 * 基于SmartRefresher的封装：下拉刷新、上拉加载更多、空视图、错误视图、重新加载
 * 1.空视图的时候只能下拉刷新，不能上拉
 * 2.错误视图的时候只能重新加载，不能上拉和下拉
 */
class RefreshUtils{
  static Widget? errorWidgetOrg;
  static Widget? emptyWidgetOrg;
  static Widget? notNetWidgetOrg;
  static Color? color_main_Org;
  static setData({Widget? errorWidget,
    Widget? emptyWidget,
    Widget? notNetWidget,
    Color? color_main,
  }){
    errorWidgetOrg=errorWidget;
    emptyWidgetOrg=emptyWidget;
    notNetWidgetOrg=notNetWidget;
    color_main_Org=color_main;
  }
  static init(
      RefreshController refreshController,@required VoidCallback? onRefresh,@required VoidCallback? onLoading,Widget childWidget,
      {
        Key? key,
        bool enablePullDown:false,
        bool enablePullUp:false,
        ScrollPhysics physics: const BouncingScrollPhysics(),
        RefreshState loadStatus:RefreshState.INIT,
        Widget? errorWidget,
        Widget? emptyWidget,
        Widget? notNetWidget,
        VoidCallback? onReLoadData,
        VoidCallback? onReFreshFunction,
        bool isCanMoreData:true,
        double footerHeight:60,
        ScrollController? scrollController,
      }){

    return SmartRefresher(
      key: key,
      scrollController: scrollController,
      controller: refreshController,
      enablePullDown: (loadStatus==RefreshState.INIT||loadStatus==RefreshState.OK||loadStatus==RefreshState.EMPTY)?enablePullDown:false,
      enablePullUp: (loadStatus==RefreshState.OK&&isCanMoreData==true)?enablePullUp:false,
      physics: physics,
      child:_setChildUi(loadStatus,childWidget, errorWidget, emptyWidget, notNetWidget,onReFreshFunction),
     header: CustomHeader(
       builder: (BuildContext context,RefreshStatus? mode){
         Widget body ;
         if(mode==RefreshStatus.idle){
           body =  Text("下拉刷新");
         }
         else if(mode==RefreshStatus.refreshing){
           body =  CupertinoActivityIndicator();
         }
         else if(mode == RefreshStatus.failed){
           body = Text("刷新再试试");
         }
         else if(mode == RefreshStatus.canRefresh){
           body = Text("释放刷新");
         }
         else{
           body = Text("刷新成功");
         }
         return Container(
           height: 55.0,
           child: Center(child:body),
         );
       },
     ),
      footer: CustomFooter(
        builder: (BuildContext context,LoadStatus? mode){
          Widget body ;
          if(mode==LoadStatus.idle){
            body =  Text("");
          }
          else if(mode==LoadStatus.loading){
            body =  CupertinoActivityIndicator();
          }
          else if(mode == LoadStatus.failed){
            body = Text("加载失败！点击重试！");
          }
          else if(mode == LoadStatus.canLoading){
            body = Text("松手,加载更多!");
          }
          else{
            body = Text("没有更多数据了!");
          }
          return Container(
            height: 30.0,
            child: Center(child:body),
          );
        },
      ),
      onRefresh: onRefresh,
      onLoading:onLoading,
    );
}

  static _setChildUi(
      RefreshState loadStatus,
      Widget childWidget,
      Widget? errorWidget,
      Widget? emptyWidget,
      Widget? notNetWidget,
      VoidCallback? onReFreshFunction
      ) {
    if(loadStatus==null){
      loadStatus=RefreshState.EMPTY;
    }
    Widget childUi=
            Container(
              child: Center(
                child: Text(''),
              ),
            );
    switch(loadStatus){
      case RefreshState.NOT_NET:
        // TODO: Handle this case.
        childUi=
            notNetWidget==null?
            Container(
              child: Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
//                    ImageUtils.loadAssetImage(
//                        DrawableUtils.neterror_img,
//                        width: PxUtils.setSize(ScreenUtil.getInstance().screenWidth*2/3),
//                        fit: BoxFit.fitWidth
//                    ),
                    Container(
                      margin: EdgeInsets.only(
                          top: PxUtils.px_5,
                      ),
                      child: Text(
                          '${StringUtils.neterror}'
                      ),
                    ),
                    SimpleButton(
                      child: Container(
                        margin: EdgeInsets.only(
                          top:PxUtils.px_39d5,
                          left:PxUtils.px_91,
                          right:PxUtils.px_91,
                        ),
                        alignment: Alignment.center,
                        height: PxUtils.px_35,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.all(Radius.circular(PxUtils.px_17d5)),
                            gradient: LinearGradient(
                                colors: [
                                  ColorUtils.color_dcbe9b,
                                  ColorUtils.color_dcbe9b,
                                ]
                            )
                        ),
                        child: Text(
                          StringUtils.clickrefresh,
                          style: TextStyle(
                              fontSize: PxUtils.px_15,
                              color: ColorUtils.color_white,
                              fontWeight: FontWeight.w500,
                              fontFamily: 'PingFang-SC-Medium'
                          ),
                        ),
                      ),
                      onPressed:(onReFreshFunction==null)?null:()=> onReFreshFunction(),
                    )
                  ],
                ),
              ),
            ):notNetWidget;
        break;
      case RefreshState.NET_ERROR:
        // TODO: Handle this case.
        childUi=
            errorWidget==null?
            Container(
              child: Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
//                    ImageUtils.loadAssetImage(
//                        DrawableUtils.neterror_img,
//                        width: PxUtils.setSize(ScreenUtil.getInstance().screenWidth*2/3),
//                        fit: BoxFit.fitWidth
//                    ),
                    Container(
                      margin: EdgeInsets.only(
                          top: PxUtils.px_5
                      ),
                      child: Text(
                          '${StringUtils.neterror}'
                      ),
                    ),
                    SimpleButton(
                      child: Container(
                        margin: EdgeInsets.only(
                          top:PxUtils.px_39d5,
                          left:PxUtils.px_91,
                          right:PxUtils.px_91,
                        ),
                        alignment: Alignment.center,
                        height: PxUtils.px_35,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.all(Radius.circular(PxUtils.px_17d5)),
                            gradient: LinearGradient(
                                colors: [
                                  ColorUtils.color_dcbe9b,
                                  ColorUtils.color_dcbe9b,
                                ]
                            )
                        ),
                        child: Text(
                          StringUtils.clickrefresh,
                          style: TextStyle(
                              fontSize: PxUtils.px_15,
                              color: ColorUtils.color_white,
                              fontWeight: FontWeight.w500,
                              fontFamily: 'PingFang-SC-Medium'
                          ),
                        ),
                      ),
                      onPressed:(onReFreshFunction==null)?null:()=> onReFreshFunction(),
                    )
                  ],
                ),
              ),
            ):errorWidget;
        break;
      case RefreshState.EMPTY:
        // TODO: Handle this case.
        childUi=
            emptyWidget==null?
            Container(
              child: Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    emptyWidgetOrg??Icon(
                        Icons.error_outline,
                      size: PxUtils.setSize(ScreenUtil.getInstance().screenWidth*1/2),
                    )
                   // ImageUtils.loadAssetImage(
                   //     DrawableUtils.nodata_img,
                   //     width: PxUtils.setSize(ScreenUtil.getInstance().screenWidth*1/2),
                   //     fit: BoxFit.fitWidth
                   // ),
                  ],
                ),
              ),
            ):emptyWidget;
        break;
      case RefreshState.OK:
        // TODO: Handle this case.
        childUi=childWidget==null?
            Container(
              child: Center(
                child: Text('有数据了'),
              ),
            ):childWidget;
        break;
      case RefreshState.INIT:
      // TODO: Handle this case.
        childUi=
            Container(
              child: Center(
                child: Text(''),
              ),
            );
        break;
      case RefreshState.LOADING:
      // TODO: Handle this case.
        childUi=
            Container(
              child: Center(
                child: Text('数据加载中...'),
              ),
            );
        break;
    }
    return childUi;
  }
}

enum RefreshState{
  NOT_NET,
  NET_ERROR,
  EMPTY,
  OK,
  INIT,
  LOADING,
}